@@ -21,8 +21,11 @@
 #define GENERICLAYOUT_H
 
 // local
+#include <coretypes.h>
 #include "abstractlayout.h"
-#include "../../liblatte2/types.h"
+#include "../data/errordata.h"
+#include "../data/viewdata.h"
+#include "../data/viewstable.h"
 
 // Qt
 #include <QObject>
@@ -47,26 +50,10 @@ class View;
 
 namespace Latte {
 namespace Layout {
-class Storage;
-}
-}
-
-namespace Latte {
-namespace Layout {
-
-struct ViewData
-{
-    int id; //view id
-    bool active; //is active
-    bool onPrimary; //on primary
-    int screenId; //explicit screen id
-    int location; //edge location
-    QList<int> systrays;
-};
 
 //! This is  views map in the following structure:
 //! SCREEN_NAME -> EDGE -> VIEWID
-typedef QHash<QString, QHash<Plasma::Types::Location, uint>> ViewsMap;
+typedef QHash<QString, QHash<Plasma::Types::Location, QList<uint>>> ViewsMap;
 
 class GenericLayout : public AbstractLayout
 {
@@ -77,20 +64,17 @@ public:
     GenericLayout(QObject *parent, QString layoutFile, QString assignedName = QString());
     ~GenericLayout() override;
 
+    QString background() const override;
+    QString textColor() const override;
+
     virtual const QStringList appliedActivities() = 0; // to move at an interface
 
     void importToCorona();
     bool initToCorona(Latte::Corona *corona);
 
     bool isActive() const; //! is loaded and running
-    virtual bool isCurrent() const;
+    virtual bool isCurrent();
     bool isWritable() const;
-    bool layoutIsBroken() const;
-
-    bool isInternalContainment(Plasma::Applet *applet) const;
-    Plasma::Containment *internalContainmentOf(Plasma::Applet *applet) const;
-
-    virtual bool configViewIsShown() const;
 
     virtual int viewsCount(int screen) const;
     virtual int viewsCount(QScreen *screen) const;
@@ -98,16 +82,28 @@ public:
 
     Type type() const override;
 
-    Latte::Corona *corona();
+    Latte::Corona *corona() const;
 
     QStringList unloadedContainmentsIds();
 
-    virtual Types::ViewType latteViewType(int containmentId) const;
-    const QList<Plasma::Containment *> *containments();
+    virtual Types::ViewType latteViewType(uint containmentId) const;
+    const QList<Plasma::Containment *> *containments() const;
+
+    bool contains(Plasma::Containment *containment) const;
+    bool containsView(const int &containmentId) const;
+    int screenForContainment(Plasma::Containment *containment);
 
     Latte::View *highestPriorityView();
-    Latte::View *viewForContainment(Plasma::Containment *containment);
-    virtual QList<Latte::View *> sortedLatteViews(QList<Latte::View *> views = QList<Latte::View *>());
+    Latte::View *viewForContainment(uint id) const;
+    Latte::View *viewForContainment(Plasma::Containment *containment) const;
+    Plasma::Containment *containmentForId(uint id) const;
+    QList<Plasma::Containment *> subContainmentsOf(uint id) const;
+
+    static bool viewAtLowerScreenPriority(Latte::View *test, Latte::View *base);
+    static bool viewAtLowerEdgePriority(Latte::View *test, Latte::View *base);
+    static QList<Latte::View *> sortedLatteViews(QList<Latte::View *> views);
+
+    QList<Latte::View *> sortedLatteViews();
     virtual QList<Latte::View *> viewsWithPlasmaShortcuts();
     virtual QList<Latte::View *> latteViews();
     ViewsMap validViewsMap(ViewsMap *occupiedMap = nullptr);
@@ -126,32 +122,43 @@ public:
 
     //! this function needs the layout to have first set the corona through initToCorona() function
     virtual void addView(Plasma::Containment *containment, bool forceOnPrimary = false, int explicitScreen = -1, Layout::ViewsMap *occupied = nullptr);
-    void copyView(Plasma::Containment *containment);
     void recreateView(Plasma::Containment *containment, bool delayed = true);
     bool latteViewExists(Plasma::Containment *containment);
 
+    Data::View newView(const Latte::Data::View &nextViewData);
+    void removeView(const Latte::Data::View &viewData);
+    void updateView(const Latte::Data::View &viewData);    
+    QString storedView(const int &containmentId); //returns temp filepath containing all view data
+    void removeOrphanedSubContainment(const int &containmentId);
+
     //! Available edges for specific view in that screen
     virtual QList<Plasma::Types::Location> availableEdgesForView(QScreen *scr, Latte::View *forView) const;
     //! All free edges in that screen
     virtual QList<Plasma::Types::Location> freeEdges(QScreen *scr) const;
     virtual QList<Plasma::Types::Location> freeEdges(int screen) const;
 
-    //! Bind this latteView and its relevant containments(including systrays)
+    //! Bind this latteView and its relevant containments(including subcontainments)
     //! to this layout. It is used for moving a Latte::View from layout to layout)
     void assignToLayout(Latte::View *latteView, QList<Plasma::Containment *> containments);
     //! Unassign that latteView from this layout (this is used for moving a latteView
     //! from layout to layout) and returns all the containments relevant to
     //! that latteView
-    QList<Plasma::Containment *> unassignFromLayout(Latte::View *latteView);
+    QList<Plasma::Containment *> unassignFromLayout(Plasma::Containment *latteContainment);
 
-    QString reportHtml(const ScreenPool *screenPool);
     QList<int> viewsScreens();
 
+    Latte::Data::ViewsTable viewsTable() const;
+
+    //! errors/warnings
+    Data::ErrorsList errors() const;
+    Data::WarningsList warnings() const;
+
 public slots:
-    Q_INVOKABLE void addNewView();
     Q_INVOKABLE int viewsWithTasks() const;
     virtual Q_INVOKABLE QList<int> qmlFreeEdges(int screen) const;  //change <Plasma::Types::Location> to <int> types
 
+    void toggleHiddenState(QString viewName, QString screenName, Plasma::Types::Location edge);
+
 signals:
     void activitiesChanged(); // to move at an interface
     void viewsCountChanged();
@@ -180,6 +187,7 @@ private slots:
     void appletCreated(Plasma::Applet *applet);
     void destroyedChanged(bool destroyed);
     void containmentDestroyed(QObject *cont);
+    void onLastConfigViewChangedFrom(Latte::View *view);
 
 private:
     //! It can be used in order for LatteViews to not be created automatically when
@@ -190,18 +198,17 @@ private:
     bool explicitDockOccupyEdge(int screen, Plasma::Types::Location location) const;
     bool primaryDockOccupyEdge(Plasma::Types::Location location) const;
 
-    bool viewAtLowerScreenPriority(Latte::View *test, Latte::View *base);
-    bool viewAtLowerEdgePriority(Latte::View *test, Latte::View *base);
-
-    bool viewDataAtLowerEdgePriority(const ViewData &test, const ViewData &base) const;
-    bool viewDataAtLowerScreenPriority(const ViewData &test, const ViewData &base) const;
-    bool viewDataAtLowerStatePriority(const ViewData &test, const ViewData &base) const;
+    bool viewDataAtLowerEdgePriority(const Latte::Data::View &test, const Latte::Data::View &base) const;
+    bool viewDataAtLowerScreenPriority(const Latte::Data::View &test, const Latte::Data::View &base) const;
+    bool viewDataAtLowerStatePriority(const Latte::Data::View &test, const Latte::Data::View &base) const;
 
     bool mapContainsId(const ViewsMap *map, uint viewId) const;
 
-    QList<int> containmentSystrays(Plasma::Containment *containment) const;
+    QList<int> subContainmentsOf(Plasma::Containment *containment) const;
 
-    QList<ViewData> sortedViewsData(const QList<ViewData> &viewsData);
+    QList<Latte::Data::View> sortedViewsData(const QList<Latte::Data::View> &viewsData);
+
+    void destroyContainment(Plasma::Containment *containment);
 
 private:
     bool m_blockAutomaticLatteViewCreation{false};
@@ -210,12 +217,12 @@ private:
 
     QStringList m_unloadedContainmentsIds;
 
-    QPointer<Storage> m_storage;
-
     //! try to avoid crashes from recreating the same views all the time
     QList<const Plasma::Containment *> m_viewsToRecreate;
 
-    friend class Storage;
+    //! Containments that are pending screen/state updates
+    Latte::Data::ViewsTable m_pendingContainmentUpdates;
+
     friend class Latte::View;
 };
 
