@@ -21,16 +21,16 @@
 #include "importer.h"
 
 // local
+#include <coretypes.h>
 #include "manager.h"
 #include "../lattecorona.h"
 #include "../screenpool.h"
 #include "../layout/abstractlayout.h"
 #include "../settings/universalsettings.h"
-#include "../liblatte2/types.h"
+#include "../tools/commontools.h"
 
 // Qt
 #include <QFile>
-#include <QTemporaryDir>
 
 // KDE
 #include <KArchive/KTar>
@@ -40,6 +40,13 @@
 #include <KLocalizedString>
 #include <KNotification>
 
+
+enum SessionType
+{
+    DefaultSession = 0,
+    AlternativeSession
+};
+
 namespace Latte {
 namespace Layouts {
 
@@ -47,6 +54,8 @@ Importer::Importer(QObject *parent)
     : QObject(parent)
 {
     m_manager = qobject_cast<Layouts::Manager *>(parent);
+
+    qDebug() << " IMPORTER, STORAGE TEMP DIR ::: " << m_storageTmpDir.path();
 }
 
 Importer::~Importer()
@@ -55,17 +64,17 @@ Importer::~Importer()
 
 bool Importer::updateOldConfiguration()
 {
-    QFile oldAppletsFile(QDir::homePath() + "/.config/lattedock-appletsrc");
+    QFile oldAppletsFile(Latte::configPath() + "/lattedock-appletsrc");
 
     if (!oldAppletsFile.exists()) {
         return false;
     }
 
     //! import standard old configuration and create the relevant layouts
-    importOldLayout(QDir::homePath() + "/.config/lattedock-appletsrc", i18n("My Layout"));
-    importOldLayout(QDir::homePath() + "/.config/lattedock-appletsrc", i18n("Alternative"), true);
+    importOldLayout(Latte::configPath() + "/lattedock-appletsrc", i18n("My Layout"));
+    importOldLayout(Latte::configPath() + "/lattedock-appletsrc", i18n("Alternative"), true);
 
-    QFile extFile(QDir::homePath() + "/.config/lattedockextrc");
+    QFile extFile(Latte::configPath() + "/lattedockextrc");
 
     //! import also the old user layouts into the new architecture
     if (extFile.exists()) {
@@ -80,7 +89,7 @@ bool Importer::updateOldConfiguration()
     }
 
     m_manager->corona()->universalSettings()->setVersion(2);
-    m_manager->corona()->universalSettings()->setCurrentLayoutName(i18n("My Layout"));
+    m_manager->corona()->universalSettings()->setSingleModeLayoutName(i18n("My Layout"));
 
     return true;
 }
@@ -106,14 +115,14 @@ bool Importer::importOldLayout(QString oldAppletsPath, QString newName, bool alt
         KConfigGroup containmentGroup = containments.group(containmentId);
 
         QString plugin = containmentGroup.readEntry("plugin", QString());
-        Types::SessionType session = (Types::SessionType)containmentGroup.readEntry("session", (int)Types::DefaultSession);
+        SessionType session = (SessionType)containmentGroup.readEntry("session", (int)DefaultSession);
 
         bool shouldImport = false;
 
-        if (plugin == "org.kde.latte.containment" && session == Types::DefaultSession && !alternative) {
+        if (plugin == "org.kde.latte.containment" && session == DefaultSession && !alternative) {
             qDebug() << containmentId << " - " << plugin << " - " << session;
             shouldImport = true;
-        } else if (plugin == "org.kde.latte.containment" && session == Types::AlternativeSession && alternative) {
+        } else if (plugin == "org.kde.latte.containment" && session == AlternativeSession && alternative) {
             qDebug() << containmentId << " - " << plugin << " - " << session;
             shouldImport = true;
         }
@@ -215,7 +224,6 @@ QStringList Importer::standardPathsFor(QString subPath, bool localfirst)
     return paths;
 }
 
-
 QString Importer::standardPath(QString subPath, bool localfirst)
 {
     QStringList paths = standardPaths(localfirst);
@@ -235,6 +243,11 @@ QString Importer::standardPath(QString subPath, bool localfirst)
     return "";
 }
 
+QString Importer::storageTmpDir() const
+{
+    return m_storageTmpDir.path();
+}
+
 QString Importer::layoutCanBeImported(QString oldAppletsPath, QString newName, QString exportDirectory)
 {
     QFile oldAppletsrc(oldAppletsPath);
@@ -253,10 +266,10 @@ QString Importer::layoutCanBeImported(QString oldAppletsPath, QString newName, Q
         return QString();
     }
 
-    QDir layoutDir(exportDirectory.isNull() ? QDir::homePath() + "/.config/latte" : exportDirectory);
+    QDir layoutDir(exportDirectory.isNull() ? layoutUserDir() : exportDirectory);
 
     if (!layoutDir.exists() && exportDirectory.isNull()) {
-        QDir(QDir::homePath() + "/.config").mkdir("latte");
+        QDir(Latte::configPath()).mkdir("latte");
     }
 
     //! set up the new layout name
@@ -357,7 +370,9 @@ bool Importer::importOldConfiguration(QString oldConfigPath, QString newName)
     }
 
     //! the old configuration contains also screen values, these must be updated also
-    KSharedConfigPtr oldScreensConfig = KSharedConfig::openConfig(screensPath);
+   /*
+    * do not use any deprecated screen ids
+    * KSharedConfigPtr oldScreensConfig = KSharedConfig::openConfig(screensPath);
     KConfigGroup m_screensGroup = KConfigGroup(oldScreensConfig, "ScreenConnectors");
 
     //restore the known ids to connector mappings
@@ -365,10 +380,10 @@ bool Importer::importOldConfiguration(QString oldConfigPath, QString newName)
         QString connector = m_screensGroup.readEntry(key, QString());
         int id = key.toInt();
 
-        if (id >= 10 && !m_manager->corona()->screenPool()->knownIds().contains(id)) {
+        if (id >= 10 && !m_manager->corona()->screenPool()->hasScreenId(id)) {
             m_manager->corona()->screenPool()->insertScreenMapping(id, connector);
         }
-    }
+    }*/
 
     return true;
 }
@@ -385,13 +400,31 @@ bool Importer::exportFullConfiguration(QString file)
         return false;
     }
 
-    archive.addLocalFile(QString(QDir::homePath() + "/.config/lattedockrc"), QStringLiteral("lattedockrc"));
+    archive.addLocalFile(QString(Latte::configPath() + "/lattedockrc"), QStringLiteral("lattedockrc"));
 
     for(const auto &layoutName : availableLayouts()) {
-        archive.addLocalFile(layoutFilePath(layoutName), QString("latte/" + layoutName + ".layout.latte"));
+        archive.addLocalFile(layoutUserFilePath(layoutName), QString("latte/" + layoutName + ".layout.latte"));
     }
 
-    //archive.addLocalDirectory(QString(QDir::homePath() + "/.config/latte"), QStringLiteral("latte"));
+    //! custom templates
+    QDir templatesDir(Latte::configPath() + "/latte/templates");
+    QStringList filters;
+    filters.append(QString("*.layout.latte"));
+    QStringList templates = templatesDir.entryList(filters, QDir::Files | QDir::Hidden | QDir::NoSymLinks);
+
+    for (int i=0; i<templates.count(); ++i) {
+        QString templatePath = templatesDir.path() + "/" + templates[i];
+        archive.addLocalFile(templatePath, QString("latte/templates/" + templates[i]));
+    }
+
+    filters.clear();
+    filters.append(QString("*.view.latte"));
+    templates = templatesDir.entryList(filters, QDir::Files | QDir::Hidden | QDir::NoSymLinks);
+
+    for (int i=0; i<templates.count(); ++i) {
+        QString templatePath = templatesDir.path() + "/" + templates[i];
+        archive.addLocalFile(templatePath, QString("latte/templates/" + templates[i]));
+    }
 
     archive.close();
 
@@ -475,7 +508,7 @@ Importer::LatteFileVersion Importer::fileVersion(QString file)
         version2LatteDir = true;
     }
 
-    if (version1applets && version1applets) {
+    if (version1applets) {
         return ConfigVersion1;
     } else if (version2rc && version2LatteDir) {
         return ConfigVersion2;
@@ -499,18 +532,28 @@ bool Importer::importHelper(QString fileName)
         return false;
     }
 
-    QString latteDirPath(QDir::homePath() + "/.config/latte");
-    QDir latteDir(latteDirPath);
+    QDir latteDir(layoutUserDir());
 
     if (latteDir.exists()) {
         latteDir.removeRecursively();
     }
 
-    archive.directory()->copyTo(QString(QDir::homePath() + "/.config"));
+    archive.directory()->copyTo(Latte::configPath());
 
     return true;
 }
 
+QString Importer::importLayout(QString fileName)
+{
+    QString newLayoutName = importLayoutHelper(fileName);
+
+    if (!newLayoutName.isEmpty()) {
+        emit newLayoutAdded(layoutUserFilePath(newLayoutName));
+    }
+
+    return newLayoutName;
+}
+
 QString Importer::importLayoutHelper(QString fileName)
 {
     LatteFileVersion version = fileVersion(fileName);
@@ -522,22 +565,21 @@ QString Importer::importLayoutHelper(QString fileName)
     QString newLayoutName = Layout::AbstractLayout::layoutName(fileName);
     newLayoutName = uniqueLayoutName(newLayoutName);
 
-    QString newPath = QDir::homePath() + "/.config/latte/" + newLayoutName + ".layout.latte";
+    QString newPath = layoutUserFilePath(newLayoutName);
     QFile(fileName).copy(newPath);
 
     QFileInfo newFileInfo(newPath);
 
     if (newFileInfo.exists() && !newFileInfo.isWritable()) {
         QFile(newPath).setPermissions(QFileDevice::ReadUser | QFileDevice::WriteUser | QFileDevice::ReadGroup | QFileDevice::ReadOther);
-    }
+    }    
 
     return newLayoutName;
-
 }
 
 QStringList Importer::availableLayouts()
 {
-    QDir layoutDir(QDir::homePath() + "/.config/latte");
+    QDir layoutDir(layoutUserDir());
     QStringList filter;
     filter.append(QString("*.layout.latte"));
     QStringList files = layoutDir.entryList(filter, QDir::Files | QDir::NoSymLinks);
@@ -566,18 +608,23 @@ QString Importer::nameOfConfigFile(const QString &fileName)
 
 bool Importer::layoutExists(QString layoutName)
 {
-    return QFile::exists(layoutFilePath(layoutName));
+    return QFile::exists(layoutUserFilePath(layoutName));
 }
 
 
-QString Importer::layoutFilePath(QString layoutName)
+QString Importer::layoutUserDir()
 {
-    return QString(QDir::homePath() + "/.config/latte/" + layoutName + ".layout.latte");
+    return QString(Latte::configPath() + "/latte");
+}
+
+QString Importer::layoutUserFilePath(QString layoutName)
+{
+    return QString(layoutUserDir() + "/" + layoutName + ".layout.latte");
 }
 
 QString Importer::uniqueLayoutName(QString name)
 {
-    int pos_ = name.lastIndexOf(QRegExp(QString("[-][0-9]+")));
+    int pos_ = name.lastIndexOf(QRegExp(QString(" - [0-9]+")));
 
     if (layoutExists(name) && pos_ > 0) {
         name = name.left(pos_);
@@ -588,7 +635,7 @@ QString Importer::uniqueLayoutName(QString name)
     QString namePart = name;
 
     while (layoutExists(name)) {
-        name = namePart + "-" + QString::number(i);
+        name = namePart + " - " + QString::number(i);
         i++;
     }
 
@@ -597,7 +644,7 @@ QString Importer::uniqueLayoutName(QString name)
 
 QStringList Importer::checkRepairMultipleLayoutsLinkedFile()
 {
-    QString linkedFilePath = QDir::homePath() + "/.config/latte/" + Layout::AbstractLayout::MultipleLayoutsName + ".layout.latte";
+    QString linkedFilePath = layoutUserFilePath(Layout::MULTIPLELAYOUTSHIDDENNAME);
     KSharedConfigPtr filePtr = KSharedConfig::openConfig(linkedFilePath);
     KConfigGroup linkedContainments = KConfigGroup(filePtr, "Containments");
 
@@ -617,9 +664,9 @@ QStringList Importer::checkRepairMultipleLayoutsLinkedFile()
     QStringList updatedLayouts;
 
     for(const auto &layoutName : linkedLayoutContainmentGroups.uniqueKeys()) {
-        if (layoutName != Layout::AbstractLayout::MultipleLayoutsName && layoutExists(layoutName)) {
+        if (layoutName != Layout::MULTIPLELAYOUTSHIDDENNAME && layoutExists(layoutName)) {
             updatedLayouts << layoutName;
-            KSharedConfigPtr layoutFilePtr = KSharedConfig::openConfig(layoutFilePath(layoutName));
+            KSharedConfigPtr layoutFilePtr = KSharedConfig::openConfig(layoutUserFilePath(layoutName));
             KConfigGroup origLayoutContainments = KConfigGroup(layoutFilePtr, "Containments");
 
             //Clear old containments
