@@ -22,15 +22,16 @@
 
 // local
 #include "lattecorona.h"
+#include "panelbackground.h"
 #include "../../layouts/importer.h"
 #include "../../view/panelshadows_p.h"
 #include "../../wm/schemecolors.h"
-#include "../../../liblatte2/commontools.h"
+#include "../../tools/commontools.h"
 
 // Qt
 #include <QDebug>
 #include <QDir>
-#include <QProcess>
+#include <QPainter>
 
 // KDE
 #include <KDirWatch>
@@ -48,8 +49,14 @@ namespace PlasmaExtended {
 
 Theme::Theme(KSharedConfig::Ptr config, QObject *parent) :
     QObject(parent),
-    m_themeGroup(KConfigGroup(config, QStringLiteral("PlasmaThemeExtended")))
+    m_themeGroup(KConfigGroup(config, QStringLiteral("PlasmaThemeExtended"))),
+    m_backgroundTopEdge(new PanelBackground(Plasma::Types::TopEdge, this)),
+    m_backgroundLeftEdge(new PanelBackground(Plasma::Types::LeftEdge, this)),
+    m_backgroundBottomEdge(new PanelBackground(Plasma::Types::BottomEdge, this)),
+    m_backgroundRightEdge(new PanelBackground(Plasma::Types::RightEdge, this))
 {
+    qmlRegisterTypes();
+
     m_corona = qobject_cast<Latte::Corona *>(parent);
 
     //! compositing tracking
@@ -72,10 +79,9 @@ Theme::Theme(KSharedConfig::Ptr config, QObject *parent) :
 
     loadConfig();
 
-    connect(this, &Theme::compositingChanged, this, &Theme::roundnessChanged);
+    connect(this, &Theme::compositingChanged, this, &Theme::updateBackgrounds);
     connect(this, &Theme::outlineWidthChanged, this, &Theme::saveConfig);
 
-    connect(&m_theme, &Plasma::Theme::themeChanged, this, &Theme::hasShadowChanged);
     connect(&m_theme, &Plasma::Theme::themeChanged, this, &Theme::load);
     connect(&m_theme, &Plasma::Theme::themeChanged, this, &Theme::themeChanged);
 }
@@ -83,7 +89,8 @@ Theme::Theme(KSharedConfig::Ptr config, QObject *parent) :
 void Theme::load()
 {
     loadThemePaths();
-    loadRoundness();
+    updateBackgrounds();
+    updateMarginsAreaValues();
 }
 
 Theme::~Theme()
@@ -96,7 +103,7 @@ Theme::~Theme()
 
 bool Theme::hasShadow() const
 {
-    return PanelShadows::self()->hasShadows();
+    return m_hasShadow;
 }
 
 bool Theme::isLightTheme() const
@@ -109,26 +116,6 @@ bool Theme::isDarkTheme() const
     return !m_isLightTheme;
 }
 
-int Theme::bottomEdgeRoundness() const
-{
-    return m_bottomEdgeRoundness;
-}
-
-int Theme::leftEdgeRoundness() const
-{
-    return m_leftEdgeRoundness;
-}
-
-int Theme::topEdgeRoundness() const
-{
-    return m_topEdgeRoundness;
-}
-
-int Theme::rightEdgeRoundness() const
-{
-    return m_rightEdgeRoundness;
-}
-
 int Theme::outlineWidth() const
 {
     return m_outlineWidth;
@@ -144,24 +131,45 @@ void Theme::setOutlineWidth(int width)
     emit outlineWidthChanged();
 }
 
-float Theme::bottomEdgeMaxOpacity() const
+int Theme::marginsAreaTop() const
 {
-    return m_bottomEdgeMaxOpacity;
+    return m_marginsAreaTop;
 }
 
-float Theme::leftEdgeMaxOpacity() const
+int Theme::marginsAreaLeft() const
 {
-    return m_leftEdgeMaxOpacity;
+    return m_marginsAreaLeft;
 }
 
-float Theme::topEdgeMaxOpacity() const
+int Theme::marginsAreaBottom() const
 {
-    return m_topEdgeMaxOpacity;
+    return m_marginsAreaBottom;
 }
 
-float Theme::rightEdgeMaxOpacity() const
+int Theme::marginsAreaRight() const
 {
-    return m_rightEdgeMaxOpacity;
+    return m_marginsAreaRight;
+}
+
+
+PanelBackground *Theme::backgroundTopEdge() const
+{
+    return m_backgroundTopEdge;
+}
+
+PanelBackground *Theme::backgroundLeftEdge() const
+{
+    return m_backgroundLeftEdge;
+}
+
+PanelBackground *Theme::backgroundBottomEdge() const
+{
+    return m_backgroundBottomEdge;
+}
+
+PanelBackground *Theme::backgroundRightEdge() const
+{
+    return m_backgroundRightEdge;
 }
 
 WindowSystem::SchemeColors *Theme::defaultTheme() const
@@ -321,126 +329,48 @@ void Theme::updateReversedSchemeValues()
     }
 }
 
-int Theme::roundness(const QImage &svgImage, Plasma::Types::Location edge)
+void Theme::updateBackgrounds()
 {
-    int discovRow = (edge == Plasma::Types::TopEdge ? svgImage.height()-1 : 0);
-    int discovCol = (edge == Plasma::Types::LeftEdge ? svgImage.width()-1 : 0);
+    updateHasShadow();
 
-    int round{0};
-
-    int maxOpacity = qMin(qAlpha(svgImage.pixel(49,0)), 200);
-
-    if (edge == Plasma::Types::BottomEdge) {
-        m_bottomEdgeMaxOpacity = (float)maxOpacity / (float)255;
-    } else if (edge == Plasma::Types::LeftEdge) {
-        m_leftEdgeMaxOpacity = (float)maxOpacity / (float)255;
-    } else if (edge == Plasma::Types::TopEdge) {
-        m_topEdgeMaxOpacity = (float)maxOpacity / (float)255;
-    } else if (edge == Plasma::Types::RightEdge) {
-        m_rightEdgeMaxOpacity = (float)maxOpacity / (float)255;
-    }
-
-    if (edge == Plasma::Types::BottomEdge || edge == Plasma::Types::RightEdge || edge == Plasma::Types::TopEdge) {
-        //! TOPLEFT corner
-        //! first LEFT pixel found
-        QRgb *line = (QRgb *)svgImage.scanLine(discovRow);
-
-        for (int col=0; col<50; ++col) {
-            QRgb pixelData = line[col];
-
-            if (qAlpha(pixelData) < maxOpacity) {
-                discovCol++;
-                round++;
-            } else {
-                break;
-            }
-        }
-    } else if (edge == Plasma::Types::LeftEdge) {
-        //! it should be TOPRIGHT corner in that case
-        //! first RIGHT pixel found
-        QRgb *line = (QRgb *)svgImage.scanLine(discovRow);
-        for (int col=99; col>50; --col) {
-            QRgb pixelData = line[col];
-
-            if (qAlpha(pixelData) < maxOpacity) {
-                discovCol--;
-                round++;
-            } else {
-                break;
-            }
-        }
-    }
-
-    //! this needs investigation (the x2) I don't know if it is really needed
-    //! but it gives me the impression that returns better results
-    return round; ///**2*/;
+    m_backgroundTopEdge->update();
+    m_backgroundLeftEdge->update();
+    m_backgroundBottomEdge->update();
+    m_backgroundRightEdge->update();
 }
 
-void Theme::loadCompositingRoundness()
+void Theme::updateHasShadow()
 {
-    Plasma::FrameSvg *svg = new Plasma::FrameSvg(this);
+    Plasma::Svg *svg = new Plasma::Svg(this);
     svg->setImagePath(QStringLiteral("widgets/panel-background"));
-    svg->setEnabledBorders(Plasma::FrameSvg::AllBorders);
-    svg->resizeFrame(QSize(100,100));
+    svg->resize();
 
-    //! New approach
-    QPixmap pxm = svg->framePixmap();
+    QString cornerId = "shadow-topleft";
+    QImage corner = svg->image(svg->elementSize(cornerId), cornerId);
 
-    //! bottom roundness
-    if (svg->hasElementPrefix("south")) {
-        svg->setElementPrefix("south");
-        pxm = svg->framePixmap();
-    } else {
-        svg->setElementPrefix("");
-        pxm = svg->framePixmap();
+    int fullTransparentPixels = 0;
+
+    for(int c=0; c<corner.width(); ++c) {
+        for(int r=0; r<corner.height(); ++r) {
+            QRgb *line = (QRgb *)corner.scanLine(r);
+            QRgb point = line[c];
+
+            if (qAlpha(point) == 0) {
+                fullTransparentPixels++;
+            }
+        }
     }
-    m_bottomEdgeRoundness = roundness(pxm.toImage(), Plasma::Types::BottomEdge);
 
-    //! left roundness
-    if (svg->hasElementPrefix("west")) {
-        svg->setElementPrefix("west");
-        pxm = svg->framePixmap();
-    } else {
-        svg->setElementPrefix("");
-        pxm = svg->framePixmap();
-    }
-    m_leftEdgeRoundness = roundness(pxm.toImage(), Plasma::Types::LeftEdge);
+    int pixels = (corner.width() * corner.height());
 
-    //! top roundness
-    if (svg->hasElementPrefix("north")) {
-        svg->setElementPrefix("north");
-        pxm = svg->framePixmap();
-    } else {
-        svg->setElementPrefix("");
-        pxm = svg->framePixmap();
-    }
-    m_topEdgeRoundness = roundness(pxm.toImage(), Plasma::Types::TopEdge);
+    m_hasShadow = (fullTransparentPixels != pixels );
+    emit hasShadowChanged();
 
-    //! right roundness
-    if (svg->hasElementPrefix("east")) {
-        svg->setElementPrefix("east");
-        pxm = svg->framePixmap();
-    } else {
-        svg->setElementPrefix("");
-        pxm = svg->framePixmap();
-    }
-    m_rightEdgeRoundness = roundness(pxm.toImage(), Plasma::Types::RightEdge);
-
-  /*  qDebug() << " COMPOSITING MASK ::: " << svg->mask();
-    qDebug() << " COMPOSITING MASK BOUNDING RECT ::: " << svg->mask().boundingRect();*/
-    qDebug() << " COMPOSITING ROUNDNESS ::: " << m_bottomEdgeRoundness << " _ " << m_leftEdgeRoundness << " _ " << m_topEdgeRoundness << " _ " << m_rightEdgeRoundness;
+    qDebug() << "  PLASMA THEME TOPLEFT SHADOW :: pixels : " << pixels << "  transparent pixels" << fullTransparentPixels << " | HAS SHADOWS :" << m_hasShadow;
 
     svg->deleteLater();
 }
 
-void Theme::loadRoundness()
-{
-    loadCompositingRoundness();
-
-    emit maxOpacityChanged();
-    emit roundnessChanged();
-}
-
 void Theme::loadThemePaths()
 {
     m_themePath = Layouts::Importer::standardPath("plasma/desktoptheme/" + m_theme.themeName());
@@ -468,7 +398,7 @@ void Theme::loadThemePaths()
     } else {
         //! when plasma theme uses the kde colors
         //! we track when kde color scheme is changing
-        QString kdeSettingsFile = QDir::homePath() + "/.config/kdeglobals";
+        QString kdeSettingsFile = Latte::configPath() + "/kdeglobals";
 
         KDirWatch::self()->addFile(kdeSettingsFile);
 
@@ -486,116 +416,6 @@ void Theme::loadThemePaths()
 
         setOriginalSchemeFile(WindowSystem::SchemeColors::possibleSchemeFile("kdeglobals"));
     }
-
-    //! this is probably not needed at all in order to provide full transparency for all
-    //! plasma themes, so we disable it in order to confirm from user testing
-    //! that it is not needed at all
-    //parseThemeSvgFiles();
-}
-
-void Theme::parseThemeSvgFiles()
-{
-    QString origBackgroundSvgFile;
-    QString curBackgroundSvgFile = m_extendedThemeDir.path()+"/widgets/panel-background.svg";
-
-    if (QFileInfo(curBackgroundSvgFile).exists()) {
-        QDir(m_extendedThemeDir.path()+"/widgets").remove("panel-background.svg");
-    }
-
-    if (!QDir(m_extendedThemeDir.path()+"/widgets").exists()) {
-        QDir(m_extendedThemeDir.path()).mkdir("widgets");
-    }
-
-    if (QFileInfo(m_themeWidgetsPath+"/panel-background.svg").exists()) {
-        origBackgroundSvgFile = m_themeWidgetsPath+"/panel-background.svg";
-        QFile(origBackgroundSvgFile).copy(curBackgroundSvgFile);
-    } else if (QFileInfo(m_themeWidgetsPath+"/panel-background.svgz").exists()) {
-        origBackgroundSvgFile = m_themeWidgetsPath+"/panel-background.svgz";
-        QString tempBackFile = m_extendedThemeDir.path()+"/widgets/panel-background.svg.gz";
-        QFile(origBackgroundSvgFile).copy(tempBackFile);
-
-        //! Identify Plasma Desktop version
-        QProcess process;
-        process.start("gzip -d " + tempBackFile);
-        process.waitForFinished();
-        QString output(process.readAllStandardOutput());
-
-        qDebug() << "plasma theme, background extraction output ::: " << output;
-        qDebug() << "plasma theme, original background svg file was decompressed...";
-    }
-
-    if (QFileInfo(curBackgroundSvgFile).exists()) {
-        qDebug() << "plasma theme, panel background ::: " << curBackgroundSvgFile;
-    } else {
-        qDebug() << "plasma theme, panel background ::: was not found...";
-    }
-
-    //! Find panel-background transparency
-    QFile svgFile(curBackgroundSvgFile);
-    QString styleSvgStr;
-
-    if (svgFile.open(QIODevice::ReadOnly)) {
-        QTextStream in(&svgFile);
-        bool centerIdFound{false};
-        bool styleFound{false};
-
-        while (!in.atEnd() && !styleFound) {
-            QString line = in.readLine();
-
-            //! each time a rect starts then style can be reset
-            if (line.contains("<rect")) {
-                styleSvgStr = "";
-            }
-
-            //! identify the id "center
-            if (line.contains("id=\"center\"")) {
-                centerIdFound = true;
-            }
-
-            //! if valid style for center exists we can break
-            if (centerIdFound && !styleSvgStr.isEmpty()) {
-                break;
-            }
-
-            if (centerIdFound && line.contains("style=\"") ) {
-                styleSvgStr = line;
-            }
-
-            //! when end of "center" you can break
-            if (centerIdFound && line.contains("/rect>")) {
-                break;
-            }
-        }
-        svgFile.close();
-    }
-
-    if (!styleSvgStr.isEmpty()) {
-        int styleInd = styleSvgStr.indexOf("style=");
-        QString cleanedStr = styleSvgStr.remove(0, styleInd+7);
-        int endInd = cleanedStr.indexOf("\"");
-        styleSvgStr = cleanedStr.mid(0,endInd);
-
-        QStringList styleValues = styleSvgStr.split(";");
-        // qDebug() << "plasma theme, discovered svg style ::: " << styleValues;
-
-        float opacity{1};
-        float fillOpacity{1};
-
-        for (QString &value : styleValues) {
-            if (value.startsWith("opacity:")) {
-                opacity = value.remove(0,8).toFloat();
-            }
-            if (value.startsWith("fill-opacity:")) {
-                fillOpacity = value.remove(0,13).toFloat();
-            }
-        }
-
-      //  m_backgroundMaxOpacity = opacity * fillOpacity;
-
-      //  qDebug() << "plasma theme opacity :: " << m_backgroundMaxOpacity << " from : " << opacity << " * " << fillOpacity;
-    }
-
- //   emit backgroundMaxOpacityChanged();
 }
 
 void Theme::loadThemeLightness()
@@ -616,6 +436,114 @@ void Theme::loadThemeLightness()
     }
 }
 
+const CornerRegions &Theme::cornersMask(const int &radius)
+{
+    if (m_cornerRegions.contains(radius)) {
+        return m_cornerRegions[radius];
+    }
+
+    qDebug() << radius;
+    CornerRegions corners;
+
+    int axis = (2 * radius) + 2;
+    QImage cornerimage(axis, axis, QImage::Format_ARGB32);
+    QPainter painter(&cornerimage);
+    //!does not provide valid masks
+    //painter.setRenderHints(QPainter::Antialiasing);
+
+    QPen pen(Qt::black);
+    pen.setStyle(Qt::SolidLine);
+    pen.setWidth(1);
+    painter.setPen(pen);
+
+    QRect rectArea(0,0,axis,axis);
+    painter.fillRect(rectArea, Qt::white);
+    painter.drawRoundedRect(rectArea, axis, axis);
+
+    QRegion topleft;
+    for(int y=0; y<radius; ++y) {
+        QRgb *line = (QRgb *)cornerimage.scanLine(y);
+
+        QString bits;
+        int width{0};
+        for(int x=0; x<radius; ++x) {
+            QRgb point = line[x];
+
+            if (QColor(point) == Qt::black) {
+                bits = bits + "1 ";
+                width = qMax(0, x);
+                break;
+            } else {
+                bits = bits + "0 ";
+            }
+        }
+
+        if (width>0) {
+            topleft += QRect(0, y, width, 1);
+        }
+
+        qDebug()<< "  " << bits;
+    }
+    corners.topLeft = topleft;
+
+    QTransform transform;
+    transform.rotate(90);
+    corners.topRight = transform.map(corners.topLeft);
+    corners.topRight.translate(corners.topLeft.boundingRect().width(), 0);
+
+    corners.bottomRight = transform.map(corners.topRight);
+    corners.bottomRight.translate(corners.topLeft.boundingRect().width(), 0);
+
+    corners.bottomLeft = transform.map(corners.bottomRight);
+    corners.bottomLeft.translate(corners.topLeft.boundingRect().width(), 0);
+
+    //qDebug() << " reg top;: " << corners.topLeft;
+    //qDebug() << " reg topr: " << corners.topRight;
+    //qDebug() << " reg bottomr: " << corners.bottomRight;
+    //qDebug() << " reg bottoml: " << corners.bottomLeft;
+
+    m_cornerRegions[radius] = corners;
+    return m_cornerRegions[radius];
+}
+
+void Theme::updateMarginsAreaValues()
+{
+    m_marginsAreaTop = 0;
+    m_marginsAreaLeft = 0;
+    m_marginsAreaBottom = 0;
+    m_marginsAreaRight = 0;
+
+    Plasma::Svg *svg = new Plasma::Svg(this);
+    svg->setImagePath(QStringLiteral("widgets/panel-background"));
+
+    bool hasThickSeparatorMargins = svg->hasElement("thick-center");
+
+    if (hasThickSeparatorMargins) {
+        int topMargin = svg->hasElement("hint-top-margin") ? svg->elementSize("hint-top-margin").height() : 0;
+        int leftMargin = svg->hasElement("hint-left-margin") ? svg->elementSize("hint-left-margin").width() : 0;
+        int bottomMargin = svg->hasElement("hint-bottom-margin") ? svg->elementSize("hint-bottom-margin").height() : 0;
+        int rightMargin = svg->hasElement("hint-right-margin") ? svg->elementSize("hint-right-margin").width() : 0;
+
+        int thickTopMargin = svg->hasElement("thick-hint-top-margin") ? svg->elementSize("thick-hint-top-margin").height() : 0;
+        int thickLeftMargin = svg->hasElement("thick-hint-left-margin") ? svg->elementSize("thick-hint-left-margin").width() : 0;
+        int thickBottomMargin = svg->hasElement("thick-hint-bottom-margin") ? svg->elementSize("thick-hint-bottom-margin").height() : 0;
+        int thickRightMargin = svg->hasElement("thick-hint-right-margin") ? svg->elementSize("thick-hint-right-margin").width() : 0;
+
+        m_marginsAreaTop = qMax(0, thickTopMargin - topMargin);
+        m_marginsAreaLeft = qMax(0, thickLeftMargin - leftMargin);
+        m_marginsAreaBottom = qMax(0, thickBottomMargin - bottomMargin);
+        m_marginsAreaRight = qMax(0, thickRightMargin - rightMargin);
+    }
+
+    qDebug() << "PLASMA THEME MARGINS AREA ::" <<
+                m_marginsAreaTop << m_marginsAreaLeft <<
+                m_marginsAreaBottom << m_marginsAreaRight;
+
+    svg->deleteLater();
+
+    emit marginsAreaChanged();
+}
+
 void Theme::loadConfig()
 {
     setOutlineWidth(m_themeGroup.readEntry("outlineWidth", 1));
@@ -624,8 +552,17 @@ void Theme::loadConfig()
 void Theme::saveConfig()
 {
     m_themeGroup.writeEntry("outlineWidth", m_outlineWidth);
+}
 
-    m_themeGroup.sync();
+void Theme::qmlRegisterTypes()
+{
+#if QT_VERSION < QT_VERSION_CHECK(5, 14, 0)
+    qmlRegisterType<Latte::PlasmaExtended::Theme>();
+    qmlRegisterType<Latte::PlasmaExtended::PanelBackground>();
+#else
+    qmlRegisterAnonymousType<Latte::PlasmaExtended::Theme>("latte-dock", 1);
+    qmlRegisterAnonymousType<Latte::PlasmaExtended::PanelBackground>("latte-dock", 1);
+#endif
 }
 
 }
