@@ -23,6 +23,7 @@
 #include "view.h"
 #include "visibilitymanager.h"
 #include "../lattecorona.h"
+#include "../layouts/storage.h"
 
 // Qt
 #include <QMouseEvent>
@@ -40,6 +41,8 @@
 #include <Plasma/Corona>
 #include <PlasmaQuick/AppletQuickItem>
 
+#define BLOCKHIDINGTYPE "View::contextMenu()"
+
 namespace Latte {
 namespace ViewPart {
 
@@ -66,13 +69,150 @@ void ContextMenu::menuAboutToHide()
 
     m_contextMenu = 0;
 
-    if (!m_latteView->containment()->isUserConfiguring()) {
-        m_latteView->visibility()->setBlockHiding(false);
-    }
-
     emit menuChanged();
 }
 
+QPoint ContextMenu::popUpRelevantToParent(const QRect &parentItem, const QRect popUpRect)
+{
+    QPoint resultPoint;
+
+    if (m_latteView->location() == Plasma::Types::TopEdge) {
+        resultPoint.setX(parentItem.left());
+        resultPoint.setY(parentItem.bottom());
+    } else if (m_latteView->location() == Plasma::Types::BottomEdge) {
+        resultPoint.setX(parentItem.left());
+        resultPoint.setY(parentItem.top() - popUpRect.height() - 1);
+    } else if (m_latteView->location() == Plasma::Types::LeftEdge) {
+        resultPoint.setX(parentItem.right());
+        resultPoint.setY(parentItem.top());
+    } else if (m_latteView->location() == Plasma::Types::RightEdge) {
+        resultPoint.setX(parentItem.left() - popUpRect.width());
+        resultPoint.setY(parentItem.top());
+    }
+
+    return resultPoint;
+}
+
+QPoint ContextMenu::popUpRelevantToGlobalPoint(const QRect &parentItem, const QRect popUpRect)
+{
+    QPoint resultPoint;
+
+    if (m_latteView->location() == Plasma::Types::TopEdge) {
+        resultPoint.setX(popUpRect.x());
+        resultPoint.setY(popUpRect.y() + 1);
+    } else if (m_latteView->location() == Plasma::Types::BottomEdge) {
+        resultPoint.setX(popUpRect.x());
+        resultPoint.setY(popUpRect.y() - popUpRect.height() - 1);
+    } else if (m_latteView->location() == Plasma::Types::LeftEdge) {
+        resultPoint.setX(popUpRect.x() + 1);
+        resultPoint.setY(popUpRect.y());
+    } else if (m_latteView->location() == Plasma::Types::RightEdge) {
+        resultPoint.setX(popUpRect.x() - popUpRect.width() - 1);
+        resultPoint.setY(popUpRect.y());
+    }
+
+    return resultPoint;
+}
+
+QPoint ContextMenu::popUpTopLeft(Plasma::Applet *applet, const QRect popUpRect)
+{
+    PlasmaQuick::AppletQuickItem *ai = applet->property("_plasma_graphicObject").value<PlasmaQuick::AppletQuickItem *>();
+
+    QRect globalItemRect = m_latteView->absoluteGeometry();
+
+    if (ai && applet != m_latteView->containment()) {
+        QPointF appletGlobalTopLeft = ai->mapToGlobal(QPointF(ai->x(), ai->y()));
+        globalItemRect = QRect(appletGlobalTopLeft.x(), appletGlobalTopLeft.y(), ai->width(), ai->height());
+    }
+
+    int itemLength = (m_latteView->formFactor() == Plasma::Types::Horizontal ? globalItemRect.width() : globalItemRect.height());
+    int menuLength = (m_latteView->formFactor() == Plasma::Types::Horizontal ? popUpRect.width() : popUpRect.height());
+
+    if ((itemLength > menuLength)
+            || (applet == m_latteView->containment())
+            || (m_latteView && Layouts::Storage::self()->isSubContainment(m_latteView->layout(), applet)) ) {
+        return popUpRelevantToGlobalPoint(globalItemRect, popUpRect);
+    } else {
+        return popUpRelevantToParent(globalItemRect, popUpRect);
+    }
+}
+
+bool ContextMenu::mousePressEventForContainmentMenu(QQuickView *view, QMouseEvent *event)
+{
+    if (!event || !view || !m_latteView->containment()) {
+        return false;
+    }
+
+    if (m_contextMenu) {
+        m_contextMenu->close();
+        m_contextMenu = 0;
+        return false;
+    }
+
+    QString trigger = Plasma::ContainmentActions::eventToString(event);
+
+    if (trigger == "RightButton;NoModifier") {
+        Plasma::ContainmentActions *plugin = m_latteView->containment()->containmentActions().value(trigger);
+
+        if (!plugin || plugin->contextualActions().isEmpty()) {
+            event->setAccepted(false);
+            return false;
+        }
+
+        if (m_latteView->containment()) {
+            QMenu *desktopMenu = new QMenu;
+            desktopMenu->setAttribute(Qt::WA_TranslucentBackground);
+
+            if (desktopMenu->winId()) {
+                desktopMenu->windowHandle()->setTransientParent(m_latteView);
+            }
+
+            desktopMenu->setAttribute(Qt::WA_DeleteOnClose);
+            m_contextMenu = desktopMenu;
+            emit menuChanged();
+
+            auto ungrabMouseHack = [this]() {
+                if (m_latteView->mouseGrabberItem()) {
+                    m_latteView->mouseGrabberItem()->ungrabMouse();
+                }
+            };
+
+            if (QVersionNumber::fromString(qVersion()) > QVersionNumber(5, 8, 0)) {
+                QTimer::singleShot(0, this, ungrabMouseHack);
+            } else {
+                ungrabMouseHack();
+            }
+
+            emit m_latteView->containment()->contextualActionsAboutToShow();
+            addContainmentActions(desktopMenu, event);
+
+            desktopMenu->setAttribute(Qt::WA_TranslucentBackground);
+
+            QPoint globalPos = event->globalPos();
+            desktopMenu->adjustSize();
+
+            QRect popUpRect(globalPos.x(), globalPos.y(), desktopMenu->width(), desktopMenu->height());
+
+            globalPos = popUpRelevantToGlobalPoint(QRect(0,0,0,0), popUpRect);
+
+            if (desktopMenu->isEmpty()) {
+                delete desktopMenu;
+                event->accept();
+                return false;
+            }
+
+            connect(desktopMenu, SIGNAL(aboutToHide()), this, SLOT(menuAboutToHide()));
+
+            desktopMenu->popup(globalPos);
+            event->setAccepted(true);
+            return false;
+        }
+    }
+
+    return true;
+}
+
+
 bool ContextMenu::mousePressEvent(QMouseEvent *event)
 {
     //qDebug() << "Step -1 ...";
@@ -89,8 +229,6 @@ bool ContextMenu::mousePressEvent(QMouseEvent *event)
         //qDebug() << "Step 0.5 ...";
         m_contextMenu->close();
         m_contextMenu = 0;
-        emit menuChanged();
-        // PlasmaQuick::ContainmentView::mousePressEvent(event);
         return false;
     }
 
@@ -142,13 +280,13 @@ bool ContextMenu::mousePressEvent(QMouseEvent *event)
             if (ai && ai->isVisible() && appletContainsMouse) {
                 applet = ai->applet();
 
-                if (m_latteView && m_latteView->layout() && m_latteView->layout()->isInternalContainment(applet)) {
-                    Plasma::Containment *internalC = m_latteView->layout()->internalContainmentOf(applet);
+                if (m_latteView && Layouts::Storage::self()->isSubContainment(m_latteView->layout(), applet)) {
+                    Plasma::Containment *subContainment = Layouts::Storage::self()->subContainmentOf(m_latteView->layout(), applet);
 
-                    if (internalC) {
+                    if (subContainment) {
                         Plasma::Applet *internalApplet{nullptr};
 
-                        for (const Plasma::Applet *appletCont : internalC->applets()) {
+                        for (const Plasma::Applet *appletCont : subContainment->applets()) {
                             PlasmaQuick::AppletQuickItem *ai2 = appletCont->property("_plasma_graphicObject").value<PlasmaQuick::AppletQuickItem *>();
 
                             if (ai2 && ai2->isVisible() && ai2->contains(ai2->mapFromItem(m_latteView->contentItem(), event->pos()))) {
@@ -197,6 +335,7 @@ bool ContextMenu::mousePressEvent(QMouseEvent *event)
 
                 desktopMenu->setAttribute(Qt::WA_DeleteOnClose);
                 m_contextMenu = desktopMenu;
+                emit menuChanged();
 
                 //! deprecated old code that can be removed if the following plasma approach doesn't
                 //! create any issues with context menu creation in Latte
@@ -248,30 +387,15 @@ bool ContextMenu::mousePressEvent(QMouseEvent *event)
                 //in .exec before oxygen can polish it and set the following attribute
                 desktopMenu->setAttribute(Qt::WA_TranslucentBackground);
                 //end workaround
-                QPoint pos = event->globalPos();
+                QPoint globalPos = event->globalPos();
+                desktopMenu->adjustSize();
+
+                QRect popUpRect(globalPos.x(), globalPos.y(), desktopMenu->width(), desktopMenu->height());
 
                 if (applet) {
-                    //qDebug() << "6 ...";
-                    desktopMenu->adjustSize();
-
-                    if (m_latteView->screen()) {
-                        const QRect scr = m_latteView->screen()->geometry();
-                        int smallStep = 3;
-                        int x = event->globalPos().x() + smallStep;
-                        int y = event->globalPos().y() + smallStep;
-
-                        //qDebug()<<x << " - "<<y;
-
-                        if (event->globalPos().x() > scr.center().x()) {
-                            x = event->globalPos().x() - desktopMenu->width() - smallStep;
-                        }
-
-                        if (event->globalPos().y() > scr.center().y()) {
-                            y = event->globalPos().y() - desktopMenu->height() - smallStep;
-                        }
-
-                        pos = QPoint(x, y);
-                    }
+                    globalPos = popUpTopLeft(applet, popUpRect);
+                } else {
+                    globalPos = popUpRelevantToGlobalPoint(QRect(0,0,0,0), popUpRect);
                 }
 
                 //qDebug() << "7...";
@@ -284,10 +408,9 @@ bool ContextMenu::mousePressEvent(QMouseEvent *event)
                 }
 
                 connect(desktopMenu, SIGNAL(aboutToHide()), this, SLOT(menuAboutToHide()));
-                m_latteView->visibility()->setBlockHiding(true);
-                desktopMenu->popup(pos);
+
+                desktopMenu->popup(globalPos);
                 event->setAccepted(true);
-                emit menuChanged();
                 return false;
             }
 
@@ -298,7 +421,6 @@ bool ContextMenu::mousePressEvent(QMouseEvent *event)
     }
 
     //qDebug() << "10 ...";
-    emit menuChanged();
     return true;
     //  PlasmaQuick::ContainmentView::mousePressEvent(event);
 }
@@ -331,6 +453,8 @@ void ContextMenu::addAppletActions(QMenu *desktopMenu, Plasma::Applet *applet, Q
         return;
     }
 
+    desktopMenu->addSection(applet->pluginMetaData().name());
+
     for (QAction *action : applet->contextualActions()) {
         if (action) {
             desktopMenu->addAction(action);
@@ -393,7 +517,7 @@ void ContextMenu::addAppletActions(QMenu *desktopMenu, Plasma::Applet *applet, Q
     }
 
     if (m_latteView->containment()->immutability() == Plasma::Types::Mutable &&
-        (m_latteView->containment()->containmentType() != Plasma::Types::PanelContainment || m_latteView->containment()->isUserConfiguring())) {
+            (m_latteView->containment()->containmentType() != Plasma::Types::PanelContainment || m_latteView->containment()->isUserConfiguring())) {
         QAction *closeApplet = applet->actions()->action(QStringLiteral("remove"));
 
         //qDebug() << "checking for removal" << closeApplet;
@@ -415,7 +539,7 @@ void ContextMenu::addContainmentActions(QMenu *desktopMenu, QEvent *event)
     }
 
     if (m_latteView->containment()->corona()->immutability() != Plasma::Types::Mutable &&
-        !KAuthorized::authorizeAction(QStringLiteral("plasma/containment_actions"))) {
+            !KAuthorized::authorizeAction(QStringLiteral("plasma/containment_actions"))) {
         //qDebug() << "immutability";
         return;
     }
