@@ -24,8 +24,8 @@
 #include <array>
 
 // local
+#include <coretypes.h>
 #include "../plasma/quick/containmentview.h"
-#include "../../liblatte2/types.h"
 
 // Qt
 #include <QObject>
@@ -34,10 +34,12 @@
 // Plasma
 #include <Plasma/Containment>
 
+
 namespace Latte {
 class Corona;
 class View;
 namespace ViewPart {
+class FloatingGapWindow;
 class ScreenEdgeGhostWindow;
 }
 namespace WindowSystem {
@@ -51,14 +53,20 @@ namespace ViewPart {
 class VisibilityManager : public QObject
 {
     Q_OBJECT
+    Q_PROPERTY(bool hidingIsBlocked READ hidingIsBlocked NOTIFY hidingIsBlockedChanged)
 
     Q_PROPERTY(Latte::Types::Visibility mode READ mode WRITE setMode NOTIFY modeChanged)
     Q_PROPERTY(bool raiseOnDesktop READ raiseOnDesktop WRITE setRaiseOnDesktop NOTIFY raiseOnDesktopChanged)
-    Q_PROPERTY(bool raiseOnActivity READ raiseOnActivity WRITE setRaiseOnActivity NOTIFY raiseOnActivityChanged)
+    Q_PROPERTY(bool raiseOnActivity READ raiseOnActivity WRITE setRaiseOnActivity NOTIFY raiseOnActivityChanged)    
     Q_PROPERTY(bool isHidden READ isHidden WRITE setIsHidden NOTIFY isHiddenChanged)
-    Q_PROPERTY(bool blockHiding READ blockHiding WRITE setBlockHiding NOTIFY blockHidingChanged)
+    Q_PROPERTY(bool isShownFully READ isShownFully WRITE setIsShownFully NOTIFY isShownFullyChanged)
+    Q_PROPERTY(bool isBelowLayer READ isBelowLayer NOTIFY isBelowLayerChanged)    
     Q_PROPERTY(bool containsMouse READ containsMouse NOTIFY containsMouseChanged)
 
+    //! Floating Gap Window to identify mouse between the screenEdge and the view when it does not accept any other
+    //! user input
+    Q_PROPERTY(bool isFloatingGapWindowEnabled READ isFloatingGapWindowEnabled WRITE setIsFloatingGapWindowEnabled NOTIFY isFloatingGapWindowEnabledChanged)
+
     //! KWin Edges Support Options
     Q_PROPERTY(bool enableKWinEdges READ enableKWinEdges WRITE setEnableKWinEdges NOTIFY enableKWinEdgesChanged)
     Q_PROPERTY(bool supportsKWinEdges READ supportsKWinEdges NOTIFY supportsKWinEdgesChanged)
@@ -66,7 +74,12 @@ class VisibilityManager : public QObject
     Q_PROPERTY(int timerShow READ timerShow WRITE setTimerShow NOTIFY timerShowChanged)
     Q_PROPERTY(int timerHide READ timerHide WRITE setTimerHide NOTIFY timerHideChanged)
 
+    //! Struts
+    Q_PROPERTY(int strutsThickness READ strutsThickness WRITE setStrutsThickness NOTIFY strutsThicknessChanged)
+
 public:
+    static const QRect ISHIDDENMASK;
+
     explicit VisibilityManager(PlasmaQuick::ContainmentView *view);
     virtual ~VisibilityManager();
 
@@ -81,30 +94,55 @@ public:
     bool raiseOnActivity() const;
     void setRaiseOnActivity(bool enable);
 
+    bool isBelowLayer() const;
+
     bool isHidden() const;
     void setIsHidden(bool isHidden);
 
-    bool blockHiding() const;
-    void setBlockHiding(bool blockHiding);
+    bool isShownFully() const;
+    void setIsShownFully(bool fully);
+
+    bool hidingIsBlocked() const;
 
     bool containsMouse() const;
 
+    bool isFloatingGapWindowEnabled() const;
+    void setIsFloatingGapWindowEnabled(bool enabled);
+
     int timerShow() const;
     void setTimerShow(int msec);
 
     int timerHide() const;
     void setTimerHide(int msec);
 
+    bool hasBlockHidingEvent(const QString &type);
+    bool isSidebar() const;
+
     //! KWin Edges Support functions
     bool enableKWinEdges() const;
     void setEnableKWinEdges(bool enable);
 
     bool supportsKWinEdges() const;
 
+    //! Struts
+    int strutsThickness() const;
+    void setStrutsThickness(int thickness);
+
+    //! Used mostly to show / hide Sidebars
+    void toggleHiddenState();
+
 public slots:
     Q_INVOKABLE void hide();
     Q_INVOKABLE void show();
 
+    Q_INVOKABLE void setViewOnBackLayer();
+    Q_INVOKABLE void setViewOnFrontLayer();
+
+    Q_INVOKABLE void addBlockHidingEvent(const QString &type);
+    Q_INVOKABLE void removeBlockHidingEvent(const QString &type);
+
+    void initViewFlags();
+
 signals:
     void mustBeShown();
     void mustBeHide();
@@ -112,12 +150,17 @@ signals:
     void slideOutFinished();
     void slideInFinished();
 
+    void frameExtentsCleared();
     void modeChanged();
     void raiseOnDesktopChanged();
     void raiseOnActivityChanged();
+    void isBelowLayerChanged();
+    void isFloatingGapWindowEnabledChanged();
     void isHiddenChanged();
-    void blockHidingChanged();
+    void isShownFullyChanged();
+    void hidingIsBlockedChanged();
     void containsMouseChanged();
+    void strutsThicknessChanged();
     void timerShowChanged();
     void timerHideChanged();
 
@@ -129,6 +172,14 @@ private slots:
     void saveConfig();
     void restoreConfig();
 
+    void setIsBelowLayer(bool below);
+
+    void onHeadThicknessChanged();
+    void onHidingIsBlockedChanged();
+    void onIsFloatingGapWindowEnabledChanged();
+
+    void publishFrameExtents(bool forceUpdate = false); //! direct
+
     //! KWin Edges Support functions
     void updateKWinEdgesSupport();
 
@@ -143,9 +194,18 @@ private:
     void deleteEdgeGhostWindow();
     void updateGhostWindowState();
 
+    //! Floating Gap Support functions
+    void createFloatingGapWindow();
+    void deleteFloatingGapWindow();
+    bool supportsFloatingGap() const;
+
     void updateStrutsBasedOnLayoutsAndActivities(bool forceUpdate = false);
     void viewEventManager(QEvent *ev);
 
+    void checkMouseInFloatingArea();
+
+    bool windowContainsMouse();
+
     QRect acceptableStruts();
 
 private slots:
@@ -154,6 +214,11 @@ private slots:
     void dodgeMaximized();
     void updateHiddenState();
 
+    bool isValidMode() const;
+
+private:
+    void startTimerHide(const int &msec = 0);
+
 private:
     WindowSystem::AbstractWindowInterface *m_wm;
     Types::Visibility m_mode{Types::None};
@@ -162,23 +227,41 @@ private:
     QTimer m_timerShow;
     QTimer m_timerHide;
     QTimer m_timerStartUp;
+    QTimer m_timerPublishFrameExtents;
 
+    bool m_isBelowLayer{false};
     bool m_isHidden{false};
+    bool m_isFloatingGapWindowEnabled{false};
+    bool m_isShownFully{false};
     bool m_dragEnter{false};
-    bool m_blockHiding{false};
     bool m_containsMouse{false};
     bool m_raiseTemporarily{false};
     bool m_raiseOnDesktopChange{false};
     bool m_raiseOnActivityChange{false};
     bool m_hideNow{false};
 
+    //! valid on demand sidebar hidden state in order to be checked after slide-ins/outs
+    bool m_isRequestedShownSidebarOnDemand{false};
+
+    int m_frameExtentsHeadThicknessGap{0};
+    int m_timerHideInterval{700};
+    Plasma::Types::Location m_frameExtentsLocation{Plasma::Types::BottomEdge};
+
+    int m_strutsThickness{0};
+
+    QStringList m_blockHidingEvents;
+
     QRect m_publishedStruts;
+    QRect m_lastMask;
 
     //! KWin Edges
     bool m_enableKWinEdgesFromUser{true};
     std::array<QMetaObject::Connection, 1> m_connectionsKWinEdges;
     ScreenEdgeGhostWindow *m_edgeGhostWindow{nullptr};
 
+    //! Floating Gap
+    FloatingGapWindow *m_floatingGapWindow{nullptr};
+
     Latte::Corona *m_corona{nullptr};
     Latte::View *m_latteView{nullptr};
 
