@@ -21,7 +21,9 @@
 #include "schemecolors.h"
 
 // local
+#include <config-latte.h>
 #include "../layouts/importer.h"
+#include "../tools/commontools.h"
 
 // Qt
 #include <QDebug>
@@ -158,12 +160,16 @@ QString SchemeColors::possibleSchemeFile(QString scheme)
     QString tempScheme = scheme;
 
     if (scheme == "kdeglobals") {
-        QString settingsFile = QDir::homePath() + "/.config/kdeglobals";
+        QString settingsFile = Latte::configPath() + "/kdeglobals";
 
         if (QFileInfo(settingsFile).exists()) {
             KSharedConfigPtr filePtr = KSharedConfig::openConfig(settingsFile);
             KConfigGroup generalGroup = KConfigGroup(filePtr, "General");
-            tempScheme = generalGroup.readEntry("ColorScheme", "");
+#if KF5_VERSION_MINOR >= 78
+            tempScheme = generalGroup.readEntry("ColorScheme", "BreezeLight");
+#else
+            tempScheme = generalGroup.readEntry("ColorScheme", "Breeze");
+#endif
         }
     }
 
