@@ -1,3 +1,27 @@
+/*
+*  Copyright 2017-2018 Michail Vourlakos <mvourlakos@gmail.com>
+*
+*  This file is part of Latte-Dock
+*
+*  Latte-Dock is free software; you can redistribute it and/or
+*  modify it under the terms of the GNU General Public License as
+*  published by the Free Software Foundation; either version 2 of
+*  the License, or (at your option) any later version.
+*
+*  Latte-Dock is distributed in the hope that it will be useful,
+*  but WITHOUT ANY WARRANTY; without even the implied warranty of
+*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+*  GNU General Public License for more details.
+*
+*  You should have received a copy of the GNU General Public License
+*  along with this program.  If not, see <http://www.gnu.org/licenses/>.
+*/
+
+//! applets that haven't identified properly their Latte behavior and
+//! create broken results when used in Latte
+var blacklistedApplets = ["org.kde.redshiftControl",
+                          "org.kde.plasma.notifications",
+                          "menuZ"];
 
 function typeOf(obj, className){
     var name = obj.toString();
@@ -7,7 +31,7 @@ function typeOf(obj, className){
 }
 
 function reconsiderAppletIconItem(){
-    if (container.appletIconItem || !applet)
+    if (communicator.appletIconItem || !applet || communicator.indexerIsSupported)
         return;
 
     //! searching to find for that applet the first IconItem
@@ -18,9 +42,11 @@ function reconsiderAppletIconItem(){
         identifyKickOff();
     } else if (applet.pluginName === "org.kde.plasma.kicker") {
         identifyKicker();
+    } else if (applet.pluginName === "org.kde.plasma.notes") {
+        identifyNotes();
     } else if (applet.pluginName === "org.kde.plasma.simplemenu") {
         identifySimpleMenu();
-    } else if (applet.pluginName === "org.kde.plasma.userswitcher"&& !root.behaveAsPlasmaPanel && !container.lockZoom) {
+    } else if (applet.pluginName === "org.kde.plasma.userswitcher") {
         identifyUserSwitcher();
     } else if (applet.pluginName === "org.kde.comexpertise.plasma.kdeconnect.sms") {
         identifyKdeConnectSms();
@@ -31,7 +57,38 @@ function reconsiderAppletIconItem(){
     }
 }
 
+function checkAndUpdateAppletRootItem() {
+    if (appletDiscoveredRootItem || communicator.indexerIsSupported) {
+        return;
+    }
+
+    var level0 = applet.children;
+
+    for(var i=0; i<level0.length; ++i){
+        var level1 = level0[i].children;
+        if (!appletDiscoveredRootItem && level0[i].hasOwnProperty("latteBridge")) {
+            appletDiscoveredRootItem = level0[i];
+        }
+        if (appletDiscoveredRootItem) {
+            break;
+        }
+
+        for(var j=0; j<level1.length; ++j){
+            if (!appletDiscoveredRootItem && level1[j].hasOwnProperty("latteBridge")) {
+                appletDiscoveredRootItem = level1[j];
+            }
+            if (appletDiscoveredRootItem) {
+                break;
+            }
+        }
+    }
+}
+
 function identifyGeneric() {
+    if (blacklistedApplets.indexOf(applet.pluginName) >= 0) {
+        return;
+    }
+
     var level0 = applet.children;
 
     for(var i=0; i<level0.length; ++i){
@@ -41,12 +98,12 @@ function identifyGeneric() {
                 var level2 = level1[j].children;
                 for(var k=0; k<level2.length; ++k){
                     if (typeOf(level2[k], "IconItem")) {
-                        container.appletIconItem = level2[k];
+                        communicator.appletIconItem = level2[k];
                         return;
                     }
                 }
             } else if (typeOf(level1[j], "IconItem")) {
-                container.appletIconItem = level1[j];
+                communicator.appletIconItem = level1[j];
                 return;
             }
         }
@@ -66,7 +123,7 @@ function identifyKdeConnectSms()
             var level2 = level1[j].children;
             for(var k=0; k<level2.length; ++k){
                 if (typeOf(level2[k], "IconItem")) {
-                    container.appletIconItem = level2[k];
+                    communicator.appletIconItem = level2[k];
                     return;
                 }
             }
@@ -84,14 +141,12 @@ function identifyKickOff() {
     for(var i=0; i<level0.length; ++i){
         var level1 = level0[i].children;
         for(var j=0; j<level1.length; ++j){
-            if (applet.pluginName === "org.kde.plasma.kickoff") {
-                if (typeOf(level1[j], "QQuickMouseArea")) {
-                    var level2 = level1[j].children;
-                    for(var k=0; k<level2.length; ++k){
-                        if (typeOf(level2[k], "IconItem")) {
-                            container.appletIconItem = level2[k];
-                            return;
-                        }
+            if (typeOf(level1[j], "QQuickMouseArea")) {
+                var level2 = level1[j].children;
+                for(var k=0; k<level2.length; ++k){
+                    if (typeOf(level2[k], "IconItem")) {
+                        communicator.appletIconItem = level2[k];
+                        return;
                     }
                 }
             }
@@ -99,6 +154,54 @@ function identifyKickOff() {
     }
 }
 
+function findIconItem(currentChildren, steps) {
+    var nextitems = currentChildren;
+    var stepsFollowed = 0;
+
+    for (var i=0; i<steps.length; ++i) {
+        for (var j=0; j<nextitems.length; ++j) {
+            var item = nextitems[j];
+            if (typeOf(item, steps[i])) {
+                stepsFollowed++;
+                nextitems = item.children;
+            }
+        }
+
+        if (stepsFollowed === steps.length) {
+            break;
+        }
+    }
+
+    if (stepsFollowed === steps.length) {
+        for (var k=0; k<nextitems.length; ++k) {
+            var lastItem = nextitems[k];
+            if (typeOf(lastItem, "IconItem")) {
+                return lastItem;
+            }
+        }
+    }
+
+    return null;
+}
+
+function identifyNotes() {
+    var level0 = applet.parent.children;
+    var stepsInCompactRepresentation = ["QQuickMouseArea"];
+
+    for(var i=0; i<level0.length; ++i){
+        var level1 = level0[i].children;
+        for(var j=0; j<level1.length; ++j){
+            if (typeOf(level1[j], "CompactApplet")) {
+
+                var icon = findIconItem(level1[j].compactRepresentation.children, stepsInCompactRepresentation)
+                if (icon) {
+                    communicator.appletIconItem = icon;
+                    return;
+                }
+            }
+        }
+    }
+}
 
 function identifyUserSwitcher() {
     if (applet.pluginName !== "org.kde.plasma.userswitcher")
@@ -109,7 +212,7 @@ function identifyUserSwitcher() {
     for(var i=0; i<level0.length; ++i){
         var level1 = level0[i].children;
         for(var j=0; j<level1.length; ++j){
-            if (applet.pluginName === "org.kde.plasma.userswitcher" && !root.behaveAsPlasmaPanel && !container.lockZoom) {
+            if (applet.pluginName === "org.kde.plasma.userswitcher" && !root.behaveAsPlasmaPanel && !container.originalAppletBehavior) {
                 if (typeOf(level1[j], "QQuickMouseArea")) {
                     var level2 = level1[j].children;
                     for(var k=0; k<level2.length; ++k){
@@ -126,7 +229,7 @@ function identifyUserSwitcher() {
                                 }
                             }
                             if (iconIt && labelIt && !labelIt.visible){
-                                container.appletIconItem = iconIt;
+                                communicator.appletIconItem = iconIt;
                                 return;
                             }
                         }
@@ -160,11 +263,11 @@ function identifyKicker() {
                 }
 
                 if (iconIt) {
-                    container.appletIconItem = iconIt;
+                    communicator.appletIconItem = iconIt;
                 }
 
                 if (imageIt){
-                    container.appletImageItem = imageIt;
+                    communicator.appletImageItem = imageIt;
                 }
 
                 return;
@@ -193,11 +296,11 @@ function identifySimpleMenu() {
         }
 
         if (iconIt) {
-            container.appletIconItem = iconIt;
+            communicator.appletIconItem = iconIt;
         }
 
         if (imageIt){
-            container.appletImageItem = imageIt;
+            communicator.appletImageItem = imageIt;
         }
 
         return;
